BRT = LibStub("AceAddon-3.0"):NewAddon("BonusRollTracker", "AceConsole-3.0", "AceEvent-3.0");

-- enable for debug
--local verbose = true;


local ST = LibStub("ScrollingTable");
local DB;

local ADDON_NAME = "Bonus Roll Tracker";
local ADDON_VERSION = "v1.01";

local S_WORLD_BOSS = "World Bosses";
local S_LOOT_COLUMN = "Loot";
local S_BOSS_COLUMN = "Boss";
local S_COIN_COLUMN = "Coin";
local S_DATE_COLUMN = "Date";
local S_UNKNOWN = "Unknown";
local S_RESET_CONFIRM = "Are you sure you want to clear all bonus roll history?";

local BRT_DATE_WIDTH = 80;
local BRT_BOSS_WIDTH = 200;
local BRT_COIN_WIDTH = 180;
local BRT_LOOT_WIDTH = 370;
local BRT_MARGIN = 25;


local BOSS_DIFFICULTY =
{
	["3"] = "10N";			-- 10 Normal
	["4"] = "25N";			-- 25 Normal
	["5"] = "10H";			-- 10 Heroic
	["6"] = "25H";			-- 25 Heroic
	["7"] = "LFR";			-- Old LFR
	["14"] = "N";			-- WoD Normal
	["15"] = "H";			-- WoD Heroic
	["16"] = "M";			-- WoD Mythic
	["17"] = "LFR";			-- WoD LFR
}



local SCROLL_COLS =
{
	{
		["name"] = S_DATE_COLUMN;
	    ["width"] = BRT_DATE_WIDTH,
	    ["align"] = "LEFT",
	    --["color"] = { ["r"] = 0.5, ["g"] = 0.5, ["b"] = 1.0, ["a"] = 1.0 },
		["colorargs"] = nil,
		--["bgcolor"] = { ["r"] = 0.0,  ["g"] = 0.0,  ["b"] = 0.0,  ["a"] = 1.0 },	    
		["DoCellUpdate"] = nil,
	},
	{
		["name"] = S_BOSS_COLUMN,
		["width"] = BRT_BOSS_WIDTH,
		["aligh"] = "LEFT",
	},
	{
		["name"] = S_COIN_COLUMN,
		["width"] = BRT_COIN_WIDTH,
		["align"] = "LEFT",
	},
	{
		["name"] = S_LOOT_COLUMN,
		["width"] = BRT_LOOT_WIDTH,
		["align"] = "LEFT",
	}
};


local baseframe = nil;

-- Base UI frame
local baseframe = CreateFrame("Frame", "BRTMainFrame", UIParent);
baseframe:SetWidth(BRT_DATE_WIDTH + BRT_BOSS_WIDTH + BRT_COIN_WIDTH + BRT_LOOT_WIDTH + 2 * BRT_MARGIN);
baseframe:SetHeight(430);
baseframe:SetPoint("CENTER",UIParent,"CENTER",0,0);
baseframe:EnableMouse(true);
baseframe:SetResizable();
baseframe:SetMovable(true);
baseframe:SetFrameStrata("DIALOG");
baseframe:SetToplevel(true);
baseframe:SetBackdrop({
	bgFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Background",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
    tile = true, tileSize = 64, edgeSize = 12,
    insets = { left = 2, right = 1, top = 2, bottom = 2 }
})
baseframe:SetBackdropColor(0,0,0,1);
baseframe:SetBackdropBorderColor(1,1,1,0.2);
--

-- Title bar
local titleframe = CreateFrame("Frame", nil, baseframe)
titleframe:SetBackdrop({
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	tile = true, tileSize = 64, edgeSize = 0,
	insets = { left = 0, right = 0, top = 0, bottom = 0 }
})
titleframe:SetBackdropColor(0,0,0,1)
titleframe:SetHeight(22)
titleframe:EnableMouse()
titleframe:SetResizable()
titleframe:SetMovable(true)
titleframe:SetPoint("LEFT", baseframe, "TOPLEFT", 10, -15)
titleframe:SetPoint("RIGHT", baseframe, "TOPRIGHT", -10, -15)

local titletext = titleframe:CreateFontString(nil,"OVERLAY","GameFontNormalSmall")
titletext:SetPoint("CENTER", titleframe, "CENTER", 0, 1)
titletext:SetText(string.format("BonusRollTracker %s", ADDON_VERSION ) )
baseframe.titleframe = titleframe
--


-- Close button
local closebutton = CreateFrame("Button", "CloseButton", baseframe, "UIPanelCloseButton")
closebutton:SetWidth(32)
closebutton:SetHeight(32)
closebutton:SetPoint("TOPRIGHT", baseframe, "TOPRIGHT", -4, 1)
closebutton:RegisterForClicks("AnyUp")
closebutton:SetScript("OnClick", function() BRT:OnClickClose() end);


-- Reset History button
local resetbutton = CreateFrame("Button", "ResetButton", baseframe, "UIPanelButtonTemplate")
resetbutton:SetWidth(150)
resetbutton:SetHeight(25)
resetbutton:SetPoint("BOTTOMLEFT", baseframe, "BOTTOMLEFT", 8, 8)
resetbutton:SetText("Clear History")
resetbutton:RegisterForClicks("AnyUp")
resetbutton:SetScript("OnClick", function() BRT:OnClickReset() end);


-- Scrolling Table
local scrolltable = ST:CreateST(SCROLL_COLS, 20, 15, nil, baseframe);
scrolltable:EnableSelection(false);
scrolltable.frame:SetPoint("CENTER", baseframe, "CENTER", 0, -16);
baseframe.scrolltable = scrolltable;


-- Reset confirm dialog
StaticPopupDialogs["BRT_CONFIRM_RESET"] = 
{
	text = S_RESET_CONFIRM,
	button1 = "Yes",
	button2 = "No",
	OnAccept = function() BRT:ConfirmReset() end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,  -- avoid some UI taint
}

BRT:RegisterChatCommand("brt", "SlashProcessor");

function BRT:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("BonusRollTrackerDB");

	if not self.db.char.history then
		self.db.char.history = {};
	end

	BRT:Populate();
end

function BRT:OnEnable()
end

function BRT:OnDisable()
end


local latestEncounterName = nil;
local latestEncounterID = nil;
local latestEncounterDiff = nil;
local coinType = nil;
local bossName = nil;
local bossDiff = nil;

-- Coin types
local COIN_TYPE_CURRENCY =
{
	697,			-- 1 = Elder Charm of Good Fortune
	752,			-- 2 = Mogu Rune of Fate
	776,			-- 3 = Warforged Seal
	994,			-- 4 = Seal of Tempered Fate
}

function BRT:Populate()
	if scrolltable then
		local data = {};
		for i, rows in ipairs(self.db.char.history) do			
			local col_date = rows[1];
			local col_boss = rows[3];
			local col_coin = GetCurrencyLink(rows[2]);
			local col_loot = rows[5];

			local difficulty = BOSS_DIFFICULTY[tostring(rows[4])];
			if (difficulty ~= nil) then
				col_boss = col_boss .. " (" .. difficulty .. ")";
			end

			local fe = {col_date, col_boss, col_coin, col_loot};
			table.insert(data, 1, fe);
		end
		scrolltable:SetData(data, true);
	end
end

function BRT:Loot(timeStamp, coinType, bossName, bossDiff, lootName, lootSpec)
	if verbose then
		BRT:Print(timeStamp .. ": Bonus rolled " .. bossName .. " with " .. coinType .. ", looted " .. lootName);
	end

	local entry = {timeStamp, coinType, bossName, bossDiff, lootName, lootSpec};
	table.insert(self.db.char.history, entry);

	-- format DB data into Scrolling Table
	BRT:Populate();
end

function BRT:BONUS_ROLL_RESULT(event, ...)
	local rewardType, rewardLink, rewardQuantity, rewardSpecID = ...;

	local lootName = nil;
	local lootSpec = rewardSpecID;

	if (rewardType == "gold") then
		lootName = GetCoinText(rewardQuantity);
	elseif (rewardType == "item") then
		lootName = rewardLink;
	end

	if (bossName == nil) then bossName = S_UNKNOWN; end

	if (lootName ~= nil) then
		local timestamp = date("%x");
		BRT:Loot(timestamp, coinType, bossName, bossDiff, lootName, lootSpec);
	end

	latestEncounterID = nil;
	latestEncounterName = nil;
	latestEncounterDiff = nil;
end

function BRT:ENCOUNTER_START(event, ...)
	latestEncounterID = nil;
	latestEncounterName = nil;
	latestEncounterDiff = nil;
end

function BRT:ENCOUNTER_END(event, ...)
	local encounterID, encounterName, difficultyID, raidSize, endStatus = ...
	if verbose then
		BRT:Print("Encounter end: id = " .. encounterID .. ", name = " .. encounterName .. ", difficulty = " .. difficultyID);
	end

	if (endStatus == 1) then
		latestEncounterName = encounterName;
		latestEncounterID = encounterID;
		latestEncounterDiff = difficultyID;
	end
end

function BRT:BONUS_ROLL_ACTIVATE(event, ...)
	-- get coin type from BonusRollFrame
	coinType = BonusRollFrame.currencyID;

	-- get boss name
	if (IsInInstance() == true) then
		-- if we're inside an instance, use the encounter name
		bossName = latestEncounterName;
		bossDiff = latestEncounterDiff;
	else
		-- not an instance, let's just call it a World Boss
		bossName = S_WORLD_BOSS;
		bossDiff = nil;
	end
end

function BRT:SlashProcessor(input)
	if (baseframe:IsShown()) then
		baseframe:Hide();
	else
		baseframe:Show();
	end
end

function BRT:OnClickClose()
	baseframe:Hide();
end

function BRT:OnClickReset()
	StaticPopup_Show("BRT_CONFIRM_RESET");
end

function BRT:ConfirmReset()
	self.db.char.history = {};

	BRT:Populate();
end

BRT:RegisterEvent("BONUS_ROLL_RESULT");
BRT:RegisterEvent("BONUS_ROLL_ACTIVATE");
BRT:RegisterEvent("ENCOUNTER_END");
BRT:RegisterEvent("ENCOUNTER_START");

-- allow ESC to close our frame
tinsert(UISpecialFrames, "BRTMainFrame");